extends Control


@onready
var _tab_container := $TabContainer
var _extra_tab_button: Button = null
var _tab_popup_menu: PopupMenu = null


func _ready() -> void:
	_tab_popup_menu = PopupMenu.new()
	_tab_popup_menu.add_item("Settings")
	_tab_popup_menu.add_item("Quit")
	add_child(_tab_popup_menu)
	_tab_container.pre_popup_pressed.connect(func(): _tab_popup_menu.size = Vector2.ZERO)

	var menu_button := MenuButton.new()
	menu_button.flat = true
	menu_button.tooltip_text = "Menu"
	menu_button.icon = preload("res://PopupMenu.svg")

	var menu = menu_button.get_popup()
	menu.add_item("Option 1", 0)
	menu.add_item("Option 2", 1)
	menu.add_item("Option 3", 2)

	var normal_button := Button.new()
	normal_button.flat = true
	normal_button.tooltip_text = "Button"
	normal_button.icon = preload("res://Object.svg")

	_tab_container.add_button(menu_button)
	_tab_container.add_button(normal_button)

	%AddExtraTabButton.pressed.connect(_add_extra_tab_button)
	%RemoveExtraTabButton.pressed.connect(_remove_extra_tab_button)
	%FreeExtraTabButton.pressed.connect(_free_extra_tab_button)

	%ToggleTabPopup.pressed.connect(_toggle_tab_popup)

	%AlignTabsLeft.pressed.connect(_change_tabs_alignment.bind(TabBar.ALIGNMENT_LEFT))
	%AlignTabsCenter.pressed.connect(_change_tabs_alignment.bind(TabBar.ALIGNMENT_CENTER))
	%AlignTabsRight.pressed.connect(_change_tabs_alignment.bind(TabBar.ALIGNMENT_RIGHT))

	%ToggleRTLLayout.pressed.connect(_toggle_rtl_layout)

	get_viewport().size_changed.connect(_update_layout)
	_update_layout()


func _add_extra_tab_button() -> void:
	if !is_instance_valid(_extra_tab_button):
		_extra_tab_button = Button.new()
		_extra_tab_button.flat = true
		_extra_tab_button.tooltip_text = "Extra Button"
		_extra_tab_button.icon = preload("res://Button.svg")

	_tab_container.add_button(_extra_tab_button)

	%AddExtraTabButton.disabled = true
	%RemoveExtraTabButton.disabled = false
	%FreeExtraTabButton.disabled = false


func _remove_extra_tab_button() -> void:
	if _extra_tab_button:
		_tab_container.remove_button(_extra_tab_button)

	%AddExtraTabButton.disabled = false
	%RemoveExtraTabButton.disabled = true
	%FreeExtraTabButton.disabled = true


func _free_extra_tab_button() -> void:
	_extra_tab_button.queue_free()

	%AddExtraTabButton.disabled = false
	%RemoveExtraTabButton.disabled = true
	%FreeExtraTabButton.disabled = true


func _toggle_tab_popup() -> void:
	if _tab_container.get_popup():
		_tab_container.set_popup(null)
	else:
		_tab_container.set_popup(_tab_popup_menu)


func _change_tabs_alignment(alignment: TabBar.AlignmentMode) -> void:
	_tab_container.tab_alignment = alignment


func _toggle_rtl_layout() -> void:
	if layout_direction == Control.LAYOUT_DIRECTION_RTL:
		layout_direction = Control.LAYOUT_DIRECTION_LTR
	else:
		layout_direction = Control.LAYOUT_DIRECTION_RTL


func _update_layout() -> void:
	var small_layout: bool = get_viewport().size.x < 540
	%ExtraTabButton.vertical = small_layout
	%TabsAlignment.vertical = small_layout
	%HSeparator1.visible = small_layout
	%HSeparator2.visible = small_layout
	%HSeparator3.visible = small_layout
